@@ -1,19 +1,20 @@
 from __future__ import (absolute_import, print_function, division,
                         unicode_literals)
 
+import _io
 import inspect
 import json as json_module
+import logging
 import re
-
-import _io
 import six
 
 from collections import namedtuple, Sequence, Sized
 from functools import update_wrapper
 from cookies import Cookies
-from requests.utils import cookiejar_from_dict
+from requests.adapters import HTTPAdapter
 from requests.exceptions import ConnectionError
 from requests.sessions import REDIRECT_STATI
+from requests.utils import cookiejar_from_dict
 
 try:
     from requests.packages.urllib3.response import HTTPResponse
@@ -37,12 +38,16 @@ UNSET = object()
 
 Call = namedtuple('Call', ['request', 'response'])
 
+_real_send = HTTPAdapter.send
+
 _wrapper_template = """\
 def wrapper%(signature)s:
     with responses:
         return func%(funcargs)s
 """
 
+logger = logging.getLogger('responses')
+
 
 def _is_string(s):
     return isinstance(s, six.string_types)
@@ -63,7 +68,11 @@ def _is_redirect(response):
 def get_wrapped(func, wrapper_template, evaldict):
     # Preserve the argspec for the wrapped function so that testing
     # tools such as pytest can continue to use their fixture injection.
-    args, a, kw, defaults = inspect.getargspec(func)
+    if six.PY2:
+        args, a, kw, defaults = inspect.getargspec(func)
+    else:
+        args, a, kw, defaults, kwonlyargs, kwonlydefaults, annotations = \
+            inspect.getfullargspec(func)
 
     signature = inspect.formatargspec(args, a, kw, defaults)
     is_bound_method = hasattr(func, '__self__')
@@ -130,6 +139,26 @@ class BaseResponse(object):
         self.url = _ensure_url_default_path(url)
         self.call_count = 0
 
+    def __eq__(self, other):
+        if not isinstance(other, BaseResponse):
+            return False
+
+        if self.method != other.method:
+            return False
+
+        # Can't simply do a equality check on the objects directly here since __eq__ isn't
+        # implemented for regex. It might seem to work as regex is using a cache to return
+        # the same regex instances, but it doesn't in all cases.
+        self_url = self.url.pattern if isinstance(
+            self.url, re._pattern_type) else self.url
+        other_url = other.url.pattern if isinstance(
+            other.url, re._pattern_type) else other.url
+
+        return self_url == other_url
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def _url_matches_strict(self, url, other):
         url_parsed = urlparse(url)
         other_parsed = urlparse(other)
@@ -286,18 +315,21 @@ class RequestsMock(object):
 
     def __init__(self,
                  assert_all_requests_are_fired=True,
-                 response_callback=None):
+                 response_callback=None,
+                 passthru_prefixes=()):
         self._calls = CallList()
         self.reset()
         self.assert_all_requests_are_fired = assert_all_requests_are_fired
         self.response_callback = response_callback
+        self.passthru_prefixes = tuple(passthru_prefixes)
 
     def reset(self):
         self._matches = []
         self._calls.reset()
 
-    def add(self,
-            method_or_response=None,
+    def add(
+            self,
+            method=None,  # method or ``Response``
             url=None,
             body='',
             adding_headers=None,
@@ -338,17 +370,66 @@ class RequestsMock(object):
         >>>     match_querystring=True
         >>> )
         """
-        if isinstance(method_or_response, BaseResponse):
-            self._matches.append(method_or_response)
+        if isinstance(method, BaseResponse):
+            self._matches.append(method)
             return
 
         if adding_headers is not None:
             kwargs.setdefault('headers', adding_headers)
 
-        if method_or_response:
-            kwargs['method'] = method_or_response
+        self._matches.append(
+            Response(method=method, url=url, body=body, **kwargs))
 
-        self._matches.append(Response(url=url, body=body, **kwargs))
+    def add_passthru(self, prefix):
+        """
+        Register a URL prefix to passthru any non-matching mock requests to.
+
+        For example, to allow any request to 'https://example.com', but require
+        mocks for the remainder, you would add the prefix as so:
+
+        >>> responses.add_passthru('https://example.com')
+        """
+        self.passthru_prefixes += (prefix, )
+
+    def remove(self, method_or_response=None, url=None):
+        """
+        Removes a response previously added using ``add()``, identified
+        either by a response object inheriting ``BaseResponse`` or
+        ``method`` and ``url``. Removes all matching responses.
+
+        >>> response.add(responses.GET, 'http://example.org')
+        >>> response.remove(responses.GET, 'http://example.org')
+        """
+        if isinstance(method_or_response, BaseResponse):
+            response = method_or_response
+        else:
+            response = BaseResponse(method=method_or_response, url=url)
+
+        while response in self._matches:
+            self._matches.remove(response)
+
+    def replace(self,
+                method_or_response=None,
+                url=None,
+                body='',
+                *args,
+                **kwargs):
+        """
+        Replaces a response previously added using ``add()``. The signature
+        is identical to ``add()``. The response is identified using ``method``
+        and ``url``, and the first matching response is replaced.
+
+        >>> responses.add(responses.GET, 'http://example.org', json={'data': 1})
+        >>> responses.replace(responses.GET, 'http://example.org', json={'data': 2})
+        """
+        if isinstance(method_or_response, BaseResponse):
+            response = method_or_response
+        else:
+            response = Response(
+                method=method_or_response, url=url, body=body, **kwargs)
+
+        index = self._matches.index(response)
+        self._matches[index] = response
 
     def add_callback(self,
                      method,
@@ -403,6 +484,13 @@ class RequestsMock(object):
         resp_callback = self.response_callback
 
         if match is None:
+            if request.url.startswith(self.passthru_prefixes):
+                logger.info(
+                    'request.allowed-passthru', extra={
+                        'url': request.url,
+                    })
+                return _real_send(adapter, request)
+
             error_msg = 'Connection refused: {0} {1}'.format(
                 request.method, request.url)
             response = ConnectionError(error_msg)
